/*
    This script loads the given net and imports the CSV file given below in 
    'RAW' CSV mode.
    
    Afterwards the script checks if the net and the imported lesson are compatible 
    with respect to their I/O names and prints the details of the analysis into a 
    result file.
*/

string sNetFileName = "Test.mbn";       // Change this to reflect your net name
string sCsvFileName = "Test.csv";       // Change this to reflect your CSV file name


const string RESULT_FILE_NAME = "AnalysisResults.txt";
bool sResultFileOpen = false;
file sResultFile;

void main()
{
    OpenNet(sNetFileName);
    NamesFromNet();
    ImportLessonRaw(sCsvFileName);
    
    // Try to create and open result summary file
	sResultFileOpen = sResultFile.Open(RESULT_FILE_NAME, FILE_MODE_WRITE | FILE_MODE_CREATE | FILE_SHARE_DENY_NONE);
	
	if (!sResultFileOpen)
	{
       MessageBox("Unable to open results file " + RESULT_FILE_NAME + "!");
       AbortScript();
    }
    
    string resultStr;
    
    if (AnalyseCompatibility())
    {
        resultStr = "Net and CSV file are compatible \n\n";
    }
    else
    {
        resultStr = "Net and CSV file are incompatible!\n\n";
    }
    
    sResultFile.Close();
    
    resultStr += "Details can be found in summary file:\n\n" + RESULT_FILE_NAME;
    
    MessageBox(resultStr);
}


// Anylyse the compatibility between net and loaded lesson. Report results in summary file, return 
// true if compatibility is given, false if an error has been detected. 
bool AnalyseCompatibility()
{
    bool ok = CheckLessonNamesUnique();
    
    Append2SummaryFile("\n");
    
    if (!CheckLessonNamesFoundInNet())
    {
        ok = false;
    }
    
    return ok;
}

// Check if the input and output names of the current lesson names are unique
bool CheckLessonNamesUnique()
{
    Append2SummaryFile("Checking if imported lesson input and ouput names are unique:\n\n");
    
    uint count = GetLessonInputCount();
    string s;
    bool inputsUnique = true;
    bool outputsUnique = true;
        
    for (uint i = 1; i < count; i++)
    {
        GetLessonInputName(i, s);
        uint duplicateCol = FindInLesson(true, s, i + 1);
        if (duplicateCol != 0)
        {
            if (inputsUnique)
            {
                Append2SummaryFile("Duplicate input name(s) found in lesson:\n");
            }
            inputsUnique = false;
            string colNumStr = i;
            Append2SummaryFile("-Column " + colNumStr + ": \"" + s + "\" is the same as column " + duplicateCol + "\n");
        }
    }
    
    if (!inputsUnique)
    {
        Append2SummaryFile("\n");
    }
    
    count = GetLessonOutputCount();
    
    for (uint i = 1; i < count; i++)
    {
        GetLessonOutputName(i, s);
        uint duplicateCol = FindInLesson(false, s, i + 1);
        if (duplicateCol != 0)
        {
            if (outputsUnique)
            {
                Append2SummaryFile("Duplicate output name(s) found in lesson:\n");
            }
            outputsUnique = false;
            string colNumStr = i;
            Append2SummaryFile("-Column " + colNumStr + ": \"" + s + "\" is the same as column " + duplicateCol + "\n");
        }
    }
    
    bool unique = inputsUnique && outputsUnique;
    
    if (unique)
    {
        Append2SummaryFile("--> PASS\n\n");
    }
    else
    {
        Append2SummaryFile("--> FAIL\n\n");
    }
    
    return unique;
}


// Check if all lesson column names are found in the net
bool CheckLessonNamesFoundInNet()
{
    Append2SummaryFile("Checking if imported lesson input and ouput names are found in the net:\n\n");
    
    uint count = GetLessonInputCount();
    string s;
    bool inputsOk = true;
    bool outputsOk = true;
        
    for (uint i = 1; i < count; i++)
    {
        GetLessonInputName(i, s);
        uint partner = FindInNet(true, s, 1);
        if (partner == 0)
        {
            if (inputsOk)
            {
                Append2SummaryFile("One or more input column names not found in net:\n");
            }
            inputsOk = false;
            string colNumStr = i;
            Append2SummaryFile("-Column " + colNumStr + ": \"" + s + "\" not found in net\n");
        }
    }
    
    if (!inputsOk)
    {
        Append2SummaryFile("\n");
    }
    
    count = GetLessonOutputCount();
    
    for (uint i = 1; i < count; i++)
    {
        GetLessonOutputName(i, s);
        uint partner = FindInNet(false, s, 1);
        if (partner == 0)
        {
            if (outputsOk)
            {
                Append2SummaryFile("One or more output column names not found in net:\n");
            }
            outputsOk = false;
            string colNumStr = i;
            Append2SummaryFile("-Column " + colNumStr + ": \"" + s + "\" not found in net\n");
        }
    }
    
    bool ok = inputsOk && outputsOk;
    
    if (ok)
    {
        Append2SummaryFile("--> PASS\n\n");
    }
    else
    {
        Append2SummaryFile("--> FAIL\n\n");
    }
    
    return ok;
}


// Append text to the summary file if open
void Append2SummaryFile(string text)
{
	if (sResultFileOpen)
	{
		if (!sResultFile.Write(text))
		{
			sResultFile.Close();
			sResultFileOpen = false;
			MessageBox("Unable to write to results file " + RESULT_FILE_NAME + "!");
			AbortScript();
		}
	}
}

// Find the given data column name in the currently active lesson. Start searching 
// at input column 'startColumn'. If 'input' is true then input columns are searched. 
// Else output columns are searched.
// Return the number of the found column, 0 if not found.
uint FindInLesson(bool input, string &in colName, uint startColumn)
{
    uint foundCol = 0;
    uint colCount = input ? GetLessonInputCount() : GetLessonOutputCount();
    string s;
    
    if (startColumn > 0)
    {
        for (uint i = startColumn; i <= colCount; i++)
        {
            if (input)
            {
                GetLessonInputName(i, s);
            }
            else
            {
                GetLessonOutputName(i, s);
            }
            if (s == colName)
            {
                foundCol = i;
                break;
            }
        }
    }
    
    return foundCol;
}


// Find the given I/O neuron name in the currently loaded net. Start searching 
// at input neuron 'startNeuron'. If 'input' is true then input neurons are searched. 
// Else output neurons are searched.
// Return the number of the found neuron, 0 if not found.
uint FindInNet(bool input, string &in neuronName, uint startNeuron)
{
    uint foundNeuron = 0;
    uint neuronCount = input ? GetInputCount() : GetOutputCount();
    string s;
    
    if (startNeuron > 0)
    {
        for (uint i = startNeuron; i <= neuronCount; i++)
        {
            if (input)
            {
                GetInputName(i, s);
            }
            else
            {
                GetOutputName(i, s);
            }
            if (s == neuronName)
            {
                foundNeuron = i;
                break;
            }
        }
    }
    
    return foundNeuron;
}


